% Modified CBBO (Chaotic Biogeography-based Optimisation) with Linear Migration 
%  
% CODE FOR TEST BED 2
%
% DEVELOPERS:
% NATIONAL UNIVERSITY OF COLOMBIA TEAM:
% Sergio Rivera (srriverar@unal.edu.co)
% Camilo Cortes (caacortesgu@unal.edu.co)
% Sergio Contreras (sfcontrerasp@unal.edu.co)
% Maria Guzman (maguzmanp@unal.edu.co)
%
% BASED ON A MODIFICATION OF TWO WORKS:
%
%  1) Chaotic Biogeography-based Optimisation (CBBO) algorithm version 1.1               
%  Author and programmer: Seyedali Mirjalili                        
%                                                                   
%         e-Mail: ali.mirjalili@gmail.com                           
%                 seyedali.mirjalili@griffithuni.edu.au             
%                                                                   
%       Homepage: http://www.alimirjalili.com              
%                                                                   
%   S. Saremi, S. Mirjalili, A. Lewis, Biogeography-based
%   optimisation with chaos, Neural Computing and Applications, In press,
%   2014, Springer DOI: http://dx.doi.org/10.1007/s00521-014-1597-x
%
%   Saremi S, Mirjalili S (2013) Integrating chaos to biogeographybased
%   optimization algorithm. Int J Comput Commun Eng
%   2:655658, Link: http://www.ijcce.org/papers/268-CI3011.pdf 
%
%   2) Benchmark Functions Optimization Using Binary Biogeography-Based 
%      Optimization with Aleatory-Mixed Migration (BBBO-AMM) and Binary 
%      Ant-Lion Optimizer (BALO), paper under review developed by:
%      Jose Gutierrez and Sergio Rivera (srriverar@unal.edu.co)    

clear all
clc
tTotalTime=tic; % lets track total computational time
 
callDatabase % script to load the caseStudyData
noRuns = 31; % Number of trials here
% get DSA parameters from DSAparameters.m file
DSAparameters

otherParameters = setOtherParameters(caseStudyData,dsaParameters.nParticles);
% set penalties
otherParameters.ensPenalty=10000; % insufficient generation / energy not supplied
otherParameters.voltagePenalty=10000; % bus voltage violations
otherParameters.linesPenalty=10000; % rate capacity of lines violated

[lowerBounds,upperBounds] = setVariablesBounds(caseStudyData,otherParameters);


DisplayFlag = true; % Whether or not to display results during run
GenFlag = true; % whether or not to exit BBO after population becomes uniform
PopSize = dsaParameters.nParticles; % Population sizes for which to run SBBO

% Choose the test function

% ProblemFunction=@Sphere;
% ProblemFunction=@Ackley;
% ProblemFunction=@Fletcher;
% ProblemFunction=@Griewank;
% ProblemFunction=@Penalty1;
% ProblemFunction=@Penalty2;
% ProblemFunction=@Quartic;
% ProblemFunction=@Rastrigin;
% ProblemFunction=@Rosenbrock;
% ProblemFunction=@Schwefel;
ProblemFunction=@TestBed2;

%Chaotic_map_no=1; %Chebyshev
%Chaotic_map_no=2; %Circle
Chaotic_map_no=3; %Gauss/mouse
%Chaotic_map_no=4; %Iterative
%Chaotic_map_no=5; %Logistic
%Chaotic_map_no=6; %Piecewise
%Chaotic_map_no=7; %Sine
%Chaotic_map_no=8; %Singer
%Chaotic_map_no=9; %Sinusoidal
%Chaotic_map_no=10; %Tent

%You can define the number of search agents and iterations in the Init.m file
Max_iterations=10000;% This should be equal or greater than OPTIONS.Maxgen in Init.m file

ChaosVec=zeros(10,Max_iterations);
%Calculate chaos vector
for i=1:10
    ChaosVec(i,:)=chaos(i,Max_iterations,1);
end

for iRuns=1:noRuns
    
    iRuns
    tOpt=tic;
    rand('state',sum(noRuns*100*clock))% ensure stochastic indpt trials

    [ResDB(iRuns).solution, ResDB(iRuns).fitMaxVector, ...
    ResDB(iRuns).objMaxVector, ResDB(iRuns).otherParameters,cg_curve4] =...
    CBBO31_40(ProblemFunction, DisplayFlag, PopSize, GenFlag,ChaosVec(Chaotic_map_no,:),dsaParameters,caseStudyData,otherParameters,lowerBounds,upperBounds); 
    ResDB(iRuns).tOpt=toc(tOpt); % time of each trial
    ResDB(iRuns).fitMaxVector(498)
    writeBenchTables
    
end

tTotalTime=toc(tTotalTime)

