% Modified CBBO (Chaotic Biogeography-based Optimisation) with Random Sinusoidal Migration 
%  
% CODE FOR TEST BED 2
%
% DEVELOPERS:
% NATIONAL UNIVERSITY OF COLOMBIA TEAM:
% Sergio Rivera (srriverar@unal.edu.co)
% Camilo Cortes (caacortesgu@unal.edu.co)
% Sergio Contreras (sfcontrerasp@unal.edu.co)
% Maria Guzman (maguzmanp@unal.edu.co)
%
% BASED ON THE MODIFICATION OF TWO WORKS:
%
%  1) Chaotic Biogeography-based Optimisation (CBBO) algorithm version 1.1               
%  Author and programmer: Seyedali Mirjalili                        
%                                                                   
%         e-Mail: ali.mirjalili@gmail.com                           
%                 seyedali.mirjalili@griffithuni.edu.au             
%                                                                   
%       Homepage: http://www.alimirjalili.com              
%                                                                   
%   S. Saremi, S. Mirjalili, A. Lewis, Biogeography-based
%   optimisation with chaos, Neural Computing and Applications, In press,
%   2014, Springer DOI: http://dx.doi.org/10.1007/s00521-014-1597-x
%
%   Saremi S, Mirjalili S (2013) Integrating chaos to biogeographybased
%   optimization algorithm. Int J Comput Commun Eng
%   2:655658, Link: http://www.ijcce.org/papers/268-CI3011.pdf 
%
%   2) Benchmark Functions Optimization Using Binary Biogeography-Based 
%      Optimization with Aleatory-Mixed Migration (BBBO-AMM) and Binary 
%      Ant-Lion Optimizer (BALO), paper under review developed by:
%      Jose Gutierrez and Sergio Rivera (srriverar@unal.edu.co)        

function [InitFunction, CostFunction, FeasibleFunction] = TestBed2

%global otherParameters lowerBounds upperBounds dsaParameters

InitFunction = @TestBed2Init;
CostFunction = @TestBed2Cost;
FeasibleFunction = @TestBed2Feasible;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [MaxParValue, MinParValue, Population, OPTIONS] = TestBed2Init(OPTIONS,dsaParameters,caseStudyData,otherParameters,lowerBounds,upperBounds)

global MinParValue MaxParValue

%     callDatabase % script to load the caseStudyData
%     % get DSA parameters from DSAparameters.m file
%     DSAparameters
% 
%   %  global otherParameters lowerBounds upperBounds dsaParameters
%     
%     otherParameters = setOtherParameters(caseStudyData,dsaParameters.nParticles);
%     % set penalties
%     otherParameters.ensPenalty=10000; % insufficient generation / energy not supplied
%     otherParameters.voltagePenalty=10000; % bus voltage violations
%     otherParameters.linesPenalty=10000; % rate capacity of lines violated
% 
%     [lowerBounds,upperBounds] = setVariablesBounds(caseStudyData,otherParameters);

Granularity = 0.01;
MinParValue = lowerBounds;
MaxParValue = upperBounds;
% Initialize population
for popindex = 1 : OPTIONS.popsize
    %chrom = (lowerBounds+upperBounds)/2;
    chrom = lowerBounds+(rand*(upperBounds-lowerBounds));
    Population(popindex).chrom = chrom;
end

OPTIONS.OrderDependent = true;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Population,obj,fit,otherParameters] = TestBed2Cost(OPTIONS, Population, dsaParameters,caseStudyData,otherParameters,lowerBounds,upperBounds)

% Compute the cost of each member in Population

global MinParValue MaxParValue
% % %for popindex = 1 : OPTIONS.popsize
% %     %Population(popindex).cost = 0;
% %     callDatabase % script to load the caseStudyData
% %     % get DSA parameters from DSAparameters.m file
% %     DSAparameters
% % 
% %    % global otherParameters lowerBounds upperBounds dsaParameters
% %     
% % %    dsaParameters.nParticles=1;
% %     
% %     otherParameters = setOtherParameters(caseStudyData,dsaParameters.nParticles);
% %     % set penalties
% %     otherParameters.ensPenalty=10000; % insufficient generation / energy not supplied
% %     otherParameters.voltagePenalty=10000; % bus voltage violations
% %     otherParameters.linesPenalty=10000; % rate capacity of lines violated
% % 
% %     [lowerBounds,upperBounds] = setVariablesBounds(caseStudyData,otherParameters);
    fnc=dsaParameters.fnc;
    
    %chrom=Population(popindex).chrom;
    for popindex = 1 : OPTIONS.popsize
    chrom(popindex,:)=Population(popindex).chrom;
    end
    
    [fit_superorganism, obj_superorganism, superorganism, otherParameters]=feval(fnc,chrom,caseStudyData, otherParameters);
    
    for popindex = 1 : OPTIONS.popsize
    Population(popindex).chrom=superorganism(popindex,:);
    Population(popindex).cost = fit_superorganism(popindex);
    end
    
    [globalminimum,indexbest] = min(fit_superorganism);
    obj=obj_superorganism(indexbest,:);
    fit = globalminimum;
    
        otherParameters.idBestParticle = indexbest;
        otherParameters.pfFinal = otherParameters.pfDB(:,indexbest);
        otherParameters.genCostsFinal = otherParameters.genCosts(indexbest,:);
        otherParameters.loadDRcostsFinal = otherParameters.loadDRcosts(indexbest,:);
        otherParameters.v2gChargeCostsFinal = otherParameters.v2gChargeCosts(indexbest,:);
        otherParameters.v2gDischargeCostsFinal =otherParameters.v2gDischargeCosts(indexbest,:);
        otherParameters.storageChargeCostsFinal = otherParameters.storageChargeCosts(indexbest,:);
        otherParameters.storageDischargeCostsFinal = otherParameters.storageDischargeCosts(indexbest,:);
        otherParameters.stBalanceFinal = otherParameters.stBalance(indexbest,:,:);
        otherParameters.v2gBalanceFinal = otherParameters.v2gBalance(indexbest,:,:);
        otherParameters.pensVoltageUFinal =  otherParameters.pensVoltageU(indexbest,:);
        otherParameters.pensVoltageLFinal = otherParameters.pensVoltageL(indexbest,:);
        otherParameters.pensMaxSLinesFinal = otherParameters.pensMaxSLines(indexbest,:);
        otherParameters.penSlackBusFinal = otherParameters.penSlackBus(indexbest,:);
    
%     for i = 1 : OPTIONS.numVar
%         gene = Population(popindex).chrom(i);
%         x = (gene - MinParValue) / (MaxParValue - MinParValue) * 2 * 1.28 - 1.28;
%         Population(popindex).cost = Population(popindex).cost + i * x^4;
%     end
%end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Population] = TestBed2Feasible(OPTIONS, Population)

global MinParValue MaxParValue
for i = 1 : OPTIONS.popsize
    %for k = 1 : OPTIONS.numVar
        Population(i).chrom = max(Population(i).chrom, MinParValue);
        Population(i).chrom = min(Population(i).chrom, MaxParValue);
    %end
end
return;